library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- encodes inputs using the NEC IR-protocol

entity encoder is
  port ( 
    clk : in STD_LOGIC;                            -- main clock (100Mhz)
    reset : in STD_LOGIC;                          -- reset signal
	 
	 -- encoder interface
    enable : in STD_LOGIC;                         -- enable component
	 send : in STD_LOGIC;                           -- start sending the signal
    data : in STD_LOGIC_VECTOR (31 downto 0);      -- message to send
	 
	 -- ouptut signal
    ir : out STD_LOGIC;                            -- ir channel
	 

	 --DEBUGGING--
	 
	 -- display interface
	 dsp_disp1 : out STD_LOGIC_VECTOR (7 downto 0); -- first display
	 dsp_disp2 : out STD_LOGIC_VECTOR (7 downto 0); -- second display 
	 dsp_disp3 : out STD_LOGIC_VECTOR (7 downto 0); -- third display
	 dsp_enable : out STD_LOGIC;                    -- enable signal for the display
	 
	 -- leds
	 leds : OUT STD_LOGIC_VECTOR (14 downto 0)      -- leds

  );
end encoder;

architecture Behavioral of encoder is
 
  -- timing constants for the protocol
  constant ROUND_UP : INTEGER := 6000;
  constant HDR_MARK : INTEGER := 900000;
  constant HDR_SPACE : INTEGER := 450000;
  constant BIT_MARK : INTEGER := 56000;
  constant LOGICAL_ONE : INTEGER := 160000; 
  constant LOGICAL_ZERO : INTEGER := 56000; 
  
  -- states of the state machine
  type STATES is (IDLE, SEND_HDR_MARK, SEND_HDR_SPACE, SEND_BIT_MARK, SEND_ONE_SPACE, SEND_ZERO_SPACE, DONE);
  
  signal currstate : STATES := IDLE;
  signal c : INTEGER range 0 to HDR_MARK := 0;
  signal curbit : INTEGER range 0 to 32 := 0;
  signal state : STD_LOGIC_VECTOR(5 downto 0) := "000000";
  
begin

  process (clk, reset, enable, send)
  begin
    if reset = '1' or enable = '0' then
	   currstate <= IDLE;
		state <= "000000";
      curbit <= 0;
	   ir <= '0';
		c <= 0;
	 else
	   if clk = '1' and clk'Event then

		  -- debugging outputs
        case currstate is
	       when IDLE            => state <= "001000";
	  	    when SEND_HDR_MARK   => state <= "010000";
          when SEND_HDR_SPACE  => state <= "000010";
 	       when SEND_BIT_MARK   => state <= "100000";
		    when SEND_ONE_SPACE  => state <= "000100";
			 when SEND_ZERO_SPACE => state <= "000100";
		    when DONE            => state <= "000001";
        end case;

		  case currstate is
		  
		    -- start sending as soon as the component gets 
			 -- enabled
		    when IDLE => ----------------------------
			   if send = '1' then
			     currstate <= SEND_HDR_MARK;
				  ir <= '1';
				  c <= 0;
				end if;
			 
			 -- wait for HDR_MARK many ticks and then toggle 
			 -- the signal
			 when SEND_HDR_MARK => -------------------
			   if c >= HDR_MARK then
				  currstate <= SEND_HDR_SPACE;
				  ir <= '0';
              c <= 0;
				else 
				  c <= c + 1;
				end if;
				
          -- wait for HDR_SPACE many ticks and then toggle
			 -- the signal
	       when SEND_HDR_SPACE => ------------------
			   if c >= HDR_SPACE then
				  currstate <= SEND_BIT_MARK;
				  curbit <= 0;
				  ir <= '1';
				  c <= 0;
				else
				  c <= c + 1;
				end if;
				
			 -- wait for BIT_MARK many ticks, then toggle the
			 -- signal and change to the corresponding successor
			 -- state based on the current messge data
			 when SEND_BIT_MARK => -------------------
				if c >= BIT_MARK then
				  if curbit = 32 then
				    currstate <= DONE;
				    ir <= '0';
				    c <= 0;
				  elsif data(curbit) = '0' then
				    currstate <= SEND_ZERO_SPACE;
				    ir <= '0';
				    c <= 0;
				  elsif data(curbit) = '1' then
				    currstate <= SEND_ONE_SPACE;
					 ir <= '0';
					 c <= 0;
				  end if;
				else
				  c <= c + 1;
				end if;
				
			 -- wait for LOGICAL_ONE many ticks and then toggle
			 -- the signal
			 when SEND_ONE_SPACE =>
				if c >= LOGICAL_ONE then
				  currstate <= SEND_BIT_MARK;
				  curbit <= curbit + 1;
				  ir <= '1';
				  c <= 0;
				else
				  c <= c + 1;
				end if;
				
			 -- wait for LOGICAL_ZERO many ticks and then toggle
			 -- the signal
			 when SEND_ZERO_SPACE =>
				if c >= LOGICAL_ZERO then
				  currstate <= SEND_BIT_MARK;
				  curbit <= curbit + 1;
				  ir <= '1';
				  c <= 0;
				else
				  c <= c + 1;
				end if;
				
		    -- wait for ROUND_UP many ticks and then restart
			 when DONE =>
			   if c >= ROUND_UP then
				  currstate <= IDLE;
				  curbit <= 0;
				else
				  c <= c + 1;	  
				end if;  
		  
		  end case;
		end if;
	 end if;
  end process;
  
  leds(14 downto 9) <= state;
  leds(8 downto 0)  <= (others => '0');
  dsp_disp1         <= (others => '0');
  dsp_disp2         <= (others => '0');
  dsp_disp3         <= (others => '0');
  dsp_enable        <= '0';

end Behavioral;

